#
# spec file for package dtb-source
#
# Copyright (c) 2016 SUSE LINUX GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via http://bugs.opensuse.org/
#


%define kernel_version %(rpm -q --qf "%{VERSION}" kernel-source | grep -v "is not")
%define kernel_release %(rpm -q --qf "%{RELEASE}" kernel-source | grep -v "is not" | cut -d. -f 1)
Name:           dtb-source
Version:        %{kernel_version}
Release:        9.1
Summary:        Device Tree files for $MACHINES
License:        GPL-2.0
Group:          System/Boot
Url:            http://www.kernel.org/
BuildRequires:  kernel-source

%if "dtb-source" == "dtb-source"
Source99:       pre_checkin.pl
%else
BuildRequires:  cpp
BuildRequires:  dtc >= 1.4.0
BuildRequires:  xz
Requires:       kernel = %{kernel_version}
%endif

%description
Device Tree files for $MACHINES.



%prep

%build
cp %{_prefix}/src/linux/COPYING .
%if "dtb-source" != "dtb-source"
mkdir pp
PPDIR=`pwd`/pp
export DTC_FLAGS="-R 4 -p 0x1000"

cd %{_prefix}/src/linux/arch/arm/boot/dts
for dts in ; do
	target=${dts%*.dts}
	mkdir -p $PPDIR/$(dirname $target)
	cpp -x assembler-with-cpp -undef -D__DTS__ -nostdinc -I. -I./include/ -P $target.dts -o $PPDIR/$target.dts
	dtc $DTC_FLAGS -I dts -O dtb -i ./$(dirname $target) -o $PPDIR/$target.dtb $PPDIR/$target.dts
done
%endif

%define dtbdir /boot/dtb-%{kernel_version}-%{kernel_release}

%install
%if "dtb-source" != "dtb-source"

cd pp
for dts in ; do
    target=${dts%*.dts}
    install -m 700 -d %{buildroot}%{dtbdir}/$(dirname $target)
    install -m 644 $target.dtb %{buildroot}%{dtbdir}/$(dirname $target)
%ifarch aarch64
    # HACK: work around U-Boot ignoring vendor dir
    ln -s $target.dtb %{buildroot}%{dtbdir}/$(basename $target).dtb
%endif
done
cd -

%else
cat - > README <<EOF
This package only consists for convenience reason to pass openSUSE Factory
review bots. Please take a look at the dtb-%{_arch}.src.rpm instead.
EOF

%files
%doc README COPYING

%endif

%changelog
* Sat Dec 10 2016 dmueller@suse.com
- enable more armv7l boards now that the kernel is fixed,
  including RPi2 (boo#1012094)
* Wed Nov  9 2016 dmueller@suse.com
- disable bcm2836, keystone again
* Mon Oct 10 2016 afaerber@suse.de
- Symlink .dtb files to where U-Boot expects them
* Mon Oct 10 2016 afaerber@suse.de
- Disable all aarch64 subpackages apart from broadcom
* Sun Jul 31 2016 afaerber@suse.de
- Update dtb-keystone to cover the renamed keystone-*.dts (4.7),
  not just k2hk, and drop dtb-k2e, dtb-k2l packages
  * Add dtb-k2e, dtb-k2l Provides to dtb-keystone
- Add Provides for previously updated arm64 packages
- Enable arm64 altera (v4.4), nvidia, renesas (v4.5)
- Package arm64 socionext (v4.5), al, amlogic (v4.6), lg (v4.7)
- Tidy subpackage description
* Sat Jun 18 2016 afaerber@suse.de
- Re-enable bcm2836
* Wed Mar  9 2016 afaerber@suse.de
- Package arm64 altera, amd, apm, arm, broadcom, cavium, exynos,
  freescale, marvell, mediatek, qcom, rockchip, sprd, xilinx
  * Prepare arm64 nvidia, renesas
  * Rename dtb-hisilicon64 to dtb-hisilicon for consistency. The
  idea is to automate packaging in the future based on directories.
  * Drop dtb-foundation-v8 and dtb-rtsm_ve-aemv8a (use dtb-arm instead)
  * Drop dtb-thunder-88xx (use dtb-cavium instead)
  * Drop dtb-apm-mustang (use dtb-apm instead)
* Tue Feb  9 2016 dmueller@suse.com
- disable bcm2836, not yet entered Factory
* Mon Feb  8 2016 dmueller@suse.com
- fix build for non-ARM architectures
- remove builddeps from dtb-source package
* Sun Jan 31 2016 dmueller@suse.com
- add dtb-bcm2836 (RPi2 upstream version)
* Fri Jan 22 2016 stefan.bruens@rwth-aachen.de
- Do not copy kernel source but build out-of-tree
* Wed Jan 20 2016 afaerber@suse.de
- Re-enable meson8b, new in kernel 4.4
* Fri Jan 15 2016 stefan.bruens@rwth-aachen.de
- Change summary/description for bcm2835 from RPi B to RPi 1, to
  distinguish between RPi 2 B and RPi B (original version). Also
  list models (A+, B, B+) to make it easier to find the right
  package.
  Support for A+ and B+ is new in Kernel 4.4.0
* Wed Jan 13 2016 agraf@suse.com
- Add dtb-hisilicon64 package for 64bit hisilicon SoCs
* Wed Dec 16 2015 afaerber@suse.de
- Sort ARMv7 packages alphabetically by file name
  * Revert meson8b (new in v4.4)
  * Package am3517, armada-375/385/388/395, imx7*, k2e/k2l, vf500
  * Avoid packaging vf610m4-* in dtb-vf6
* Tue Dec 15 2015 afaerber@suse.de
- Package Amlogic Meson DTBs for ARMv7
* Mon Dec  7 2015 dmueller@suse.com
- add post requires for the new dtb symlink generation
* Thu Dec  3 2015 afaerber@suse.de
- Simplify multiversion naming
* Thu Dec  3 2015 afaerber@suse.de
- Mark dtb subpackages as multiversion
* Sun Nov 15 2015 afaerber@suse.de
- Install to kernel-specific directory
  Prepares for multiversion dtb packages and extlinux support.
  * If /boot/dtb is not a real directory, symlink it for back compat
- pre_checkin.pl: Run format_spec_file (for whitespace cleanup)
* Thu Oct 15 2015 matwey.kornilov@gmail.com
- Add armv7l dtbs:
  * dtb-am57xx
* Sun Jul 26 2015 afaerber@suse.de
- Add armv7l dtbs:
  * dtb-sun8i
  * dtb-sun9i
* Fri Jul 17 2015 afaerber@suse.de
- Add armv7l dtbs:
  * dtb-qcom (qcom-apq8064-ifc6410 new in 3.16)
  * dtb-zynq (zynq-parallella new in 3.17)
* Wed Feb 18 2015 dmueller@suse.com
- add thunderx dtbs
- adjust for kernel 3.19
- require kernel version to to trigger automatic rebuild
* Mon Sep  1 2014 dmueller@suse.com
- add armv7l vexpress dtbs
* Mon May 19 2014 afaerber@suse.de
- Add armv7l dtbs:
  * dtb-vf6
  * dtb-tegra114
  * dtb-tegra124
- Clean up pre_checkin.pl to avoid changing between tabs and spaces
* Thu Apr 17 2014 guillaume@opensuse.org
- Add aarch64 dtbs:
  * dtb-apm-mustang
  * dtb-foundation-v8
  * dtb-rtsm_ve-aemv8a
* Thu Apr  3 2014 dmueller@suse.com
- build against kernel-source instead of a local copy
  of kernel sources
* Thu Apr  3 2014 dmueller@suse.com
- update pre_checkin.pl: generate all DTB's from a single
  spec file per architecture.
* Thu Apr  3 2014 guillaume@opensuse.org
- Add Rockchip RK3XXX dtbs
* Thu Apr  3 2014 guillaume@opensuse.org
- Fix dtb-keystone by using 'k2hk-evm' as only keystone board
* Thu Apr  3 2014 guillaume@opensuse.org
- Update to 3.14
* Sat Mar  8 2014 matwey.kornilov@gmail.com
- Update to 3.13.6
  +  Remove am33xx.patch.
* Wed Jan  8 2014 matwey.kornilov@gmail.com
- Add am33xx.patch: improve support Beagle Bone Black
* Fri Jan  3 2014 agraf@suse.com
- Expose bigger groups of devices to catch new ones more
  easily
* Fri Jan  3 2014 agraf@suse.com
- Add i.MX 53 dtbs
* Tue Dec 17 2013 guillaume@opensuse.org
- Add dtb-am335x which supports Beagle Bone,
  Beagle Bone Black, TI AM335x EVM and TI AM335x EVM-SK
* Tue Dec 17 2013 guillaume@opensuse.org
- Update to 3.12.5
* Thu Nov 21 2013 guillaume@opensuse.org
- Fix pre_checkin.pl script to update *.changes hardlinks even if
  files exist before (-f option)
* Thu Nov 21 2013 guillaume@opensuse.org
- Add chromebook 5250 support
* Thu Nov 14 2013 guillaume@opensuse.org
- Add Arndaleboard 5250 support
* Fri Sep 27 2013 dmueller@suse.com
- make dtb-source buildable
- change *.changes to be hardlinks
- install COPYING
* Thu Sep 26 2013 guillaume@opensuse.org
- Initial release - Version 3.11.1
